## ----setup, include=TRUE, message=FALSE, warning=FALSE-------------------
rm(list=ls())
library(spBayes)
library(fields)
library(MBA)
library(geoR)
library(raster)
library(leaflet)
library(viridis)


## ------------------------------------------------------------------------
data(COmonthlyMet)

y <- CO.tmin.MAM.climate
coords <- CO.loc
elev <- CO.elev

missing <- is.na(y)

y.ho <- y[missing]
coords.ho <- coords[missing,]
elev.ho <- elev[missing]

y <- y[!missing]
coords <- coords[!missing,]
elev <- elev[!missing]


## ------------------------------------------------------------------------
##make a design matrix for regression analysis
dat <- data.frame(y, coords, elev)

pal <- colorNumeric(palette = "YlOrRd", domain = y)

##https://rstudio.github.io/leaflet/
leaflet(dat) %>% addCircleMarkers(lng = ~lon, lat = ~lat, col = ~pal(y),
                                  stroke = FALSE, fillOpacity = 0.75,
                                  popup=~paste("Temp C:", round(y,2))) %>%
    addProviderTiles("Esri.WorldImagery", group="Satellite") %>%
    addProviderTiles("Stamen.Terrain", group="Terrain") %>%
    addProviderTiles("Esri.WorldTopoMap", group="Topographical") %>%
    addLegend("bottomright", pal = pal, values = ~y, title = "Temp C" , opacity = 0.75) %>% 
    addLayersControl(
        baseGroup = c("Topographical", "Satellite","Terrain")
    )



## ------------------------------------------------------------------------
##LM residual EDA
lm.m <- lm(y ~ elev, data=dat)
summary(lm.m)

lm.resids <- resid(lm.m)

d.max <- max(iDist(coords))
d.max

##variogram 
par(mfrow=c(1,2))
v <- variog(coords=coords, data=y, uvec=(seq(0, 0.75*d.max, length=10)))
plot(v)

v <- variog(coords=coords, data=lm.resids, uvec=(seq(0, 0.5*d.max, length=10)))
plot(v)


## ------------------------------------------------------------------------
##surfaces
par(mfrow=c(1,2))
quilt.plot(coords[,1], coords[,2], lm.resids, col=viridis(100), main="LM residuals")

resid.surf <- mba.surf(cbind(coords, lm.resids), no.X=200, no.Y=200, extend=TRUE)$xyz.est
image.plot(resid.surf, col=viridis(100), main="LM residuals\n(interpolated surface)")


## ------------------------------------------------------------------------
n.samples <- 2500

cov.model <- "exponential"
starting <- list("phi"=3/(0.5*d.max), "sigma.sq"=1, "tau.sq"=1.5)
tuning <- list("phi"=0.1, "sigma.sq"=0.05, "tau.sq"=0.05)
priors <- list("beta.Flat", "phi.Unif"=list(3/d.max, 3/(0.01*d.max)), "sigma.sq.IG"=list(2, 1), "tau.sq.IG"=c(2, 1.5))
 
n.report <- 500

sp.m <- spSVC(y ~ elev, coords=c("lon","lat"), data=dat, starting=starting,
              tuning=tuning, priors=priors, cov.model=cov.model,
              n.samples=n.samples, n.report=n.report, n.omp.threads=3)

plot(sp.m$p.theta.samples)


## ------------------------------------------------------------------------

burn.in <- 0.5*n.samples

sp.m <- spRecover(sp.m, start=burn.in, thin=2, n.omp.threads = 3)

round(summary(sp.m$p.theta.recover.samples)$quantiles[,c(3,1,5)],2)

round(summary(sp.m$p.beta.recover.samples)$quantiles[,c(3,1,5)],2)


## ------------------------------------------------------------------------
quants <- function(x){quantile(x, prob=c(0.5,0.025,0.975))}

w.summary <- apply(sp.m$p.w.recover.samples, 1, quants)

##Plot posterior median surface
par(mfrow=c(1,2))
resid.surf <- mba.surf(cbind(coords, lm.resids), no.X=200, no.Y=200, extend=TRUE)$xyz.est
image.plot(resid.surf, col=viridis(100), main="lm() resids")

w.mu.surf <- mba.surf(cbind(coords, w.summary[1,]), no.X=200, no.Y=200, extend=TRUE)$xyz.est
image.plot(w.mu.surf, col=viridis(100), main="spSVC()\nspatial random effects, w")


## ------------------------------------------------------------------------
##Load elevation (DEM grid)
data(RMelevation)

##Extract pixel centroids and predictor where prediction is desired 
pred.coords <- as.matrix(expand.grid(RMelevation$x, RMelevation$y))
pred.elev <- as.vector(RMelevation$z)
pred.X <- cbind(1, pred.elev)

pred.obj <- spPredict(sp.m, pred.covars=pred.X, pred.coords=pred.coords, thin=25, n.report=5, n.omp.threads=4)

y.post.pred.mean <- apply(pred.obj$p.y.predictive.samples, 1, mean)
y.post.pred.mean.r <- rasterFromXYZ(cbind(pred.coords,y.post.pred.mean))

##Compare spSVC() and lm() predictions
y.mean <- pred.X%*%coefficients(lm.m)
y.lm.pred.mean.r <- rasterFromXYZ(cbind(pred.coords,y.mean))

diff.r <- rasterFromXYZ(cbind(pred.coords, y.mean-y.post.pred.mean))

par(mfrow=c(1,3))
plot(y.post.pred.mean.r, main="Post pred mean")
plot(y.lm.pred.mean.r, main="lm pred mean")
plot(diff.r, main="diff")

